package com.qmth.distributed.print.business.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ZipUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.boot.core.enums.Platform;
import com.qmth.distributed.print.business.bean.auth.AuthBean;
import com.qmth.distributed.print.business.bean.auth.ExpireTimeBean;
import com.qmth.distributed.print.business.bean.dto.OrgDto;
import com.qmth.distributed.print.business.bean.dto.PrivilegeCacheDto;
import com.qmth.distributed.print.business.bean.result.LoginResult;
import com.qmth.distributed.print.business.bean.result.PrivilegeResult;
import com.qmth.distributed.print.business.bean.result.RolePrivilegeResult;
import com.qmth.distributed.print.business.config.DictionaryConfig;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.*;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.execute.AsyncCreatePdfTempleteService;
import com.qmth.distributed.print.business.util.*;
import com.qmth.distributed.print.common.contant.SpringContextHolder;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.common.util.SessionUtil;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.security.NoSuchAlgorithmException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 公共服务service impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/26
 */
@Service
public class CommonServiceImpl implements CommonService {
    private final static Logger log = LoggerFactory.getLogger(CommonServiceImpl.class);

    @Resource
    CacheService cacheService;

    @Resource
    SysUserRoleService sysUserRoleService;

    @Resource
    SysPrivilegeService sysPrivilegeService;

    @Resource
    SysRolePrivilegeService sysRolePrivilegeService;

    @Resource
    SysRoleService sysRoleService;

    @Resource
    TBSessionService tbSessionService;

    @Resource
    RedisUtil redisUtil;

    @Resource
    DictionaryConfig dictionaryConfig;

    @Resource
    OssUtil ossUtil;

    @Autowired
    private ExamTaskService examTaskService;

    @Autowired
    private ExamTaskDetailService examTaskDetailService;

    @Autowired
    private ExamDetailService examDetailService;

    @Autowired
    private SysOrgService sysOrgService;

    @Autowired
    private BasicAttachmentService basicAttachmentService;

    @Resource
    CommonService commonService;

    @Autowired
    private ExamDetailCourseService examDetailCourseService;

    @Autowired
    private TBTaskService tbTaskService;

    @Autowired
    private AsyncCreatePdfTempleteService asyncCreatePdfTempleteService;

    @Autowired
    private ConvertUtil convertUtil;

    @Autowired
    private ExamPrintPlanService examPrintPlanService;

    /**
     * 新增用户权限
     *
     * @param sysUser
     * @param roleId
     */
    @Override
    public void addUserRolePrivilege(SysUser sysUser, Long roleId) {
        List<SysRolePrivilege> sysRolePrivilegeList = cacheService.rolePrivilegeCache(roleId);
        List<SysUserRole> sysUserRoleList = new ArrayList<>();
        sysRolePrivilegeList.forEach(s -> {
            sysUserRoleList.add(new SysUserRole(sysUser.getId(), s.getRoleId(), s.getPrivilegeId()));
        });
        sysUserRoleService.saveBatch(sysUserRoleList);
    }

    /**
     * 获取用户角色权限
     *
     * @param userId
     * @return
     */
    @Override
    public List<SysUserRole> getUserRolePrivilege(Long userId) {
        QueryWrapper<SysUserRole> sysUserRoleQueryWrapper = new QueryWrapper<>();
        sysUserRoleQueryWrapper.lambda().eq(SysUserRole::getUserId, userId)
                .eq(SysUserRole::getEnable, true);
        return sysUserRoleService.list(sysUserRoleQueryWrapper);
    }

    /**
     * 获取角色权限
     *
     * @param roleId
     * @return
     */
    @Override
    public List<SysRolePrivilege> getRolePrivilege(Long roleId) {
        QueryWrapper<SysRolePrivilege> sysRolePrivilegeQueryWrapper = new QueryWrapper<>();
        sysRolePrivilegeQueryWrapper.lambda().eq(SysRolePrivilege::getRoleId, roleId);
        return sysRolePrivilegeService.list(sysRolePrivilegeQueryWrapper);
    }

    /**
     * 获取用户权限
     *
     * @param userId
     * @return
     */
    @Override
    public AuthBean getUserAuth(Long userId) {
        AuthBean authBean = null;
        try {
            CacheService cacheService = SpringContextHolder.getBean(CacheService.class);
            SysUser user = cacheService.userCache(userId);
            if (Objects.isNull(user)) {
                throw ExceptionResultEnum.USER_NO_DATA.exception();
            }
            //查询用户角色和权限
            List<SysUserRole> sysUserRoleList = cacheService.userRolePrivilegeCache(user.getId());
            if (Objects.nonNull(sysUserRoleList) && sysUserRoleList.size() > 0) {
                Set<Long> roleIds = new HashSet<>();
                Set<Long> privilegeIds = new HashSet<>();
                sysUserRoleList.forEach(s -> {
                    roleIds.add(s.getRoleId());
                    privilegeIds.add(s.getPrivilegeId());
                });
                QueryWrapper<SysRole> sysRoleQueryWrapper = new QueryWrapper<>();
                sysRoleQueryWrapper.lambda().in(SysRole::getId, roleIds)
                        .eq(SysRole::getEnable, true);
                List<SysRole> sysRoleList = sysRoleService.list(sysRoleQueryWrapper);
                int count = Objects.nonNull(sysRoleList) && sysRoleList.size() > 0 ? (int) sysRoleList.stream().filter(s -> s.getType() == RoleTypeEnum.ADMIN).count() : 0;
                QueryWrapper<SysPrivilege> sysPrivilegeQueryWrapper = new QueryWrapper<>();
                if (count > 0) {//超级系统管理员
                    sysPrivilegeQueryWrapper.lambda().eq(SysPrivilege::getType, PrivilegeEnum.URL)
                            .eq(SysPrivilege::getProperty, PrivilegePropertyEnum.AUTH);
                    List<SysPrivilege> sysPrivilegeList = sysPrivilegeService.list(sysPrivilegeQueryWrapper);
                    authBean = new AuthBean(sysRoleList, sysPrivilegeList.stream().map(s -> s.getUrl()).collect(Collectors.toSet()));
                } else {
                    BasicSchool tbSchool = Objects.nonNull(user.getSchoolId()) ? cacheService.schoolCache(user.getSchoolId()) : null;
                    SysOrg org = Objects.nonNull(user.getOrgId()) ? cacheService.orgCache(user.getOrgId()) : null;
                    sysPrivilegeQueryWrapper.lambda().in(SysPrivilege::getId, privilegeIds)
                            .eq(SysPrivilege::getProperty, PrivilegePropertyEnum.AUTH);
                    List<SysPrivilege> sysPrivilegeList = sysPrivilegeService.list(sysPrivilegeQueryWrapper);
                    authBean = new AuthBean(sysRoleList, sysPrivilegeList.stream().map(s -> s.getUrl()).collect(Collectors.toSet()), tbSchool, org);
                }
            }
        } catch (Exception e) {
            log.error("请求出错", e);
            throw ExceptionResultEnum.ERROR.exception("添加用户鉴权缓存失败");
        }
        return authBean;
    }

    /**
     * 获取鉴权url
     *
     * @param privilegePropertyEnum
     * @return
     */
    @Override
    public List<String> getPrivilegeUrl(PrivilegePropertyEnum privilegePropertyEnum) {
        QueryWrapper<SysPrivilege> sysPrivilegeQueryWrapper = new QueryWrapper<>();
        sysPrivilegeQueryWrapper.lambda().eq(SysPrivilege::getType, PrivilegeEnum.URL)
                .eq(SysPrivilege::getProperty, privilegePropertyEnum)
                .eq(SysPrivilege::getEnable, true);
        List<SysPrivilege> sysPrivilegeList = sysPrivilegeService.list(sysPrivilegeQueryWrapper);
        return Objects.nonNull(sysPrivilegeList) && sysPrivilegeList.size() > 0 ? sysPrivilegeList.stream().map(s -> s.getUrl()).collect(Collectors.toList()) : null;
    }

    /**
     * 删除用户信息
     *
     * @param userId
     */
    @Override
    public void removeUserInfo(Long userId) throws NoSuchAlgorithmException {
        AuthBean authBean = cacheService.userAuthCache(userId);
        if (Objects.isNull(authBean)) {
            throw ExceptionResultEnum.NOT_LOGIN.exception();
        }
        for (Platform p : Platform.values()) {
            String sessionId = SessionUtil.digest(userId, Math.abs(authBean.getRoleList().stream().map(s -> s.getType()).collect(Collectors.toSet()).hashCode()), p.name());
            tbSessionService.removeById(sessionId);
            redisUtil.deleteUserSession(sessionId);
        }
        cacheService.removeUserCache(userId);
        cacheService.removeUserAuthCache(userId);
    }

    /**
     * 修改角色状态
     *
     * @param roleIds
     * @param enable
     */
    @Override
    public void updateRoleEnable(Long[] roleIds, Boolean enable) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        List<SysRole> dbRoleList = sysRoleService.listByIds(Arrays.asList(roleIds));
        if (Objects.isNull(dbRoleList) || dbRoleList.size() == 0) {
            throw ExceptionResultEnum.ROLE_NO_DATA.exception();
        }

        //修改角色
        UpdateWrapper<SysRole> sysRoleUpdateWrapper = new UpdateWrapper<>();
        sysRoleUpdateWrapper.lambda().in(SysRole::getId, roleIds)
                .set(SysRole::getEnable, enable)
                .set(SysRole::getUpdateId, requestUser.getId())
                .set(SysRole::getUpdateTime, System.currentTimeMillis());
        sysRoleService.update(sysRoleUpdateWrapper);

        //修改角色权限
        UpdateWrapper<SysRolePrivilege> sysRolePrivilegeUpdateWrapper = new UpdateWrapper<>();
        sysRolePrivilegeUpdateWrapper.lambda().in(SysRolePrivilege::getRoleId, roleIds)
                .set(SysRolePrivilege::getEnable, enable);
        sysRolePrivilegeService.update(sysRolePrivilegeUpdateWrapper);

        //修改用户角色权限
        UpdateWrapper<SysUserRole> sysUserRoleUpdateWrapper = new UpdateWrapper<>();
        sysUserRoleUpdateWrapper.lambda().in(SysUserRole::getRoleId, roleIds)
                .set(SysUserRole::getEnable, enable);
        sysUserRoleService.update(sysUserRoleUpdateWrapper);

        List<Long> updateRoleIds = new ArrayList<>();
        for (SysRole dbRole : dbRoleList) {
            //如果数据库的状态和修改的状态不同
            if (Objects.nonNull(dbRole.getEnable()) && dbRole.getEnable().booleanValue() != enable.booleanValue()) {
                //更新角色权限
                cacheService.updateRolePrivilegeCache(dbRole.getId());
                updateRoleIds.add(dbRole.getId());
            }
        }
        //更新用户权限状态
        QueryWrapper<SysUserRole> sysUserRoleQueryWrapper = new QueryWrapper<>();
        sysUserRoleQueryWrapper.lambda().eq(SysUserRole::getRoleId, updateRoleIds);
        List<SysUserRole> sysUserRoleList = sysUserRoleService.list(sysUserRoleQueryWrapper);
        Set<Long> userIds = sysUserRoleList.stream().map(s -> s.getUserId()).collect(Collectors.toSet());
        for (Long l : userIds) {
            cacheService.updateUserRolePrivilegeCache(l);
            cacheService.updateUserAuthCache(l);
        }
    }

    /**
     * 查询用户或者角色权限信息
     *
     * @param privilegeCacheDtoList
     * @return
     */
    @Override
    public RolePrivilegeResult getUserOrRolePrivilege(List<PrivilegeCacheDto> privilegeCacheDtoList) {
        LinkedMultiValueMap<Long, PrivilegeResult> buttonsLinkedMultiValueMap = new LinkedMultiValueMap<>(), linksLinkedMultiValueMap = new LinkedMultiValueMap<>();
        List<PrivilegeCacheDto> buttonsFilterList = new ArrayList<>(), linksFilterList = new ArrayList<>();

        for (PrivilegeCacheDto privilegeCacheDto : privilegeCacheDtoList) {
            if (privilegeCacheDto.getPrivilegeType() == PrivilegeEnum.BUTTON) {
                buttonsFilterList.add(privilegeCacheDto);
                buttonsLinkedMultiValueMap.add(privilegeCacheDto.getParentId(), new PrivilegeResult(privilegeCacheDto));
            } else if (privilegeCacheDto.getPrivilegeType() == PrivilegeEnum.LINK) {
                linksFilterList.add(privilegeCacheDto);
                linksLinkedMultiValueMap.add(privilegeCacheDto.getParentId(), new PrivilegeResult(privilegeCacheDto));
            }
        }

        privilegeCacheDtoList.removeAll(buttonsFilterList);
        privilegeCacheDtoList.removeAll(linksFilterList);

        RolePrivilegeResult rolePrivilegeResult = new RolePrivilegeResult();
        List<PrivilegeResult> privilegeResultList = new ArrayList<>();
        for (PrivilegeCacheDto privilegeCacheDto : privilegeCacheDtoList) {
            rolePrivilegeResult.setInfo(privilegeCacheDto);
            PrivilegeResult privilegeResult = new PrivilegeResult(privilegeCacheDto);
            privilegeResult.setButtons(buttonsLinkedMultiValueMap.get(privilegeResult.getId()));
            privilegeResult.setLinks(linksLinkedMultiValueMap.get(privilegeResult.getId()));
            privilegeResultList.add(privilegeResult);
        }
        rolePrivilegeResult.setPrivileges(privilegeResultList);
        return rolePrivilegeResult;
    }

    /**
     * 文件预览
     *
     * @param path
     * @return
     */
    @Override
    public String filePreview(String path) {
        String url = null;
        JSONObject jsonObject = JSONObject.parseObject(path);
        String attachmentType = (String) jsonObject.get(SystemConstant.TYPE);
        String filePath = (String) jsonObject.get(SystemConstant.PATH);
        UploadFileEnum uploadFileEnum = UploadFileEnum.valueOf((String) jsonObject.get(SystemConstant.UPLOAD_TYPE));
        if (Objects.equals(attachmentType, SystemConstant.LOCAL)) {
            url = SystemConstant.HTTP + dictionaryConfig.sysDomain().getFileHost() + File.separator + filePath;
        } else {
            if (uploadFileEnum == UploadFileEnum.PAPER) {
                url = ossUtil.getPrivateUrl(filePath);
            } else {
                url = dictionaryConfig.aliYunOssDomain().getUrl() + File.separator + filePath;
            }
        }
        return url;
    }

    /**
     * 文件预览
     *
     * @param path     附件路径
     * @param type     保存类型：本地、OSS
     * @param isExpire url是否带过期时间
     * @return
     */
    @Override
    public String filePreviewByPathAndType(String path, String type, Boolean isExpire) {
        if (StringUtils.isBlank(path)) {
            return null;
        }

        String pathUrl;
        if (Objects.equals(type, SystemConstant.LOCAL)) {
            pathUrl = SystemConstant.HTTP + dictionaryConfig.sysDomain().getFileHost() + File.separator + path;
        } else {
            if (isExpire) {
                pathUrl = ossUtil.getPrivateUrl(path);
            } else {
                pathUrl = dictionaryConfig.aliYunOssDomain().getUrl() + File.separator + path;
            }

        }
        return pathUrl;
    }

    /**
     * 文件预览
     *
     * @param attachmentId 附件路径
     * @param isExpire     url是否带过期时间
     * @return
     */
    @Override
    public Map<String, String> filePreviewByAttachmentId(Long attachmentId, Boolean isExpire) {
        BasicAttachment attachment = basicAttachmentService.getById(attachmentId);
        if (attachment == null) {
            return null;
        }

        Map<String, String> map = new HashMap<>();

        String pathUrl;
        JSONObject jsonObject = JSONObject.parseObject(attachment.getPath());
        String attachmentType = (String) jsonObject.get(SystemConstant.TYPE);
        String filePath = (String) jsonObject.get(SystemConstant.PATH);
        if (Objects.equals(attachmentType, SystemConstant.LOCAL)) {
            pathUrl = SystemConstant.HTTP + dictionaryConfig.sysDomain().getFileHost() + File.separator + filePath;
        } else {
            if (isExpire) {
                pathUrl = ossUtil.getPrivateUrl(filePath);
            } else {
                pathUrl = dictionaryConfig.aliYunOssDomain().getUrl() + File.separator + filePath;
            }

        }
        map.put("url", pathUrl);
        map.put("md5", attachment.getMd5());
        return map;
    }

    @Override
    public String readFileContent(String path) {
        // 解析path
        JSONObject object = JSONObject.parseObject(path);
        String filePath = object.getString(SystemConstant.PATH);
        String type = object.getString(SystemConstant.TYPE);
        if (filePath.endsWith(SystemConstant.HTML_PREFIX) || filePath.endsWith(SystemConstant.FTL_PREFIX)) {
            StringBuffer sb = new StringBuffer();
            try {
                InputStream fis;
                if (type.equals(SystemConstant.OSS)) {
                    byte[] bytes = ossUtil.ossDownload(filePath);
                    fis = new ByteArrayInputStream(bytes);
                } else {
                    String url = SystemConstant.HTTP + dictionaryConfig.sysDomain().getFileHost() + File.separator + filePath;
                    File file = new File(url);
                    if (!file.exists()) {
                        throw ExceptionResultEnum.ERROR.exception("附件文件不存在");
                    }
                    fis = new FileInputStream(file);
                }
                BufferedReader br = new BufferedReader(new InputStreamReader(fis));
                String s;
                while ((s = br.readLine()) != null) {
                    sb.append(s).append("\n");
                }
            } catch (IOException e) {
                throw ExceptionResultEnum.ERROR.exception("文件读取异常");
            }
            return sb.toString();
        }
        return null;
    }

    /**
     * 获取文件流
     *
     * @param path
     * @param pdf
     * @return
     */
    @Override
    public InputStream getFileInputStream(String path, boolean pdf) throws IOException {
        InputStream inputStream = null;
        JSONObject jsonObject = JSONObject.parseObject(path);
        String attachmentType = (String) jsonObject.get(SystemConstant.TYPE);
        String filePath = pdf ? (String) jsonObject.get(SystemConstant.PDF_PATH) : (String) jsonObject.get(SystemConstant.PATH);
        if (Objects.equals(attachmentType, SystemConstant.OSS)) {
            byte[] data = ossUtil.ossDownload(filePath);
            inputStream = new ByteArrayInputStream(data);
        } else {
            StringJoiner localPath = new StringJoiner("").add(SystemConstant.TEMP_FILES_DIR).add(File.separator).add(filePath);
            inputStream = new FileInputStream(new File(localPath.toString()));
        }
        return inputStream;
    }

    /**
     * 获取文件数组
     *
     * @param path
     * @param pdf
     * @return
     * @throws IOException
     */
    @Override
    public byte[] getFileByte(String path, boolean pdf) throws IOException {
        byte[] data = null;
        JSONObject jsonObject = JSONObject.parseObject(path);
        String attachmentType = (String) jsonObject.get(SystemConstant.TYPE);
        String filePath = pdf ? (String) jsonObject.get(SystemConstant.PDF_PATH) : (String) jsonObject.get(SystemConstant.PATH);
        if (Objects.equals(attachmentType, SystemConstant.OSS)) {
            data = ossUtil.ossDownload(filePath);
        } else {
            StringJoiner localPath = new StringJoiner("").add(SystemConstant.TEMP_FILES_DIR).add(File.separator).add(filePath);
            InputStream inputStream = new FileInputStream(new File(localPath.toString()));
            ByteArrayOutputStream ou = new ByteArrayOutputStream();
            IOUtils.copy(inputStream, ou);
            data = ou.toByteArray();
        }
        return data;
    }

    /**
     * 获取文件
     *
     * @param path
     * @param pdf
     * @return
     * @throws IOException
     */
    @Override
    public File getFile(String path, boolean pdf) throws IOException {
        File file = null;
        JSONObject jsonObject = JSONObject.parseObject(path);
        String attachmentType = (String) jsonObject.get(SystemConstant.TYPE);
        String filePath = pdf ? (String) jsonObject.get(SystemConstant.PDF_PATH) : (String) jsonObject.get(SystemConstant.PATH);
        StringJoiner localPath = new StringJoiner("").add(SystemConstant.TEMP_FILES_DIR).add(File.separator).add(filePath);
        if (Objects.equals(attachmentType, SystemConstant.OSS)) {
            file = ossUtil.ossDownload(filePath, localPath.toString());
        } else {
            file = new File(localPath.toString());
        }
        return file;
    }

    @Override
    public File copyFile(String rootPath, String fileName, BasicAttachment attachment) {
        JSONObject object = JSONObject.parseObject(attachment.getPath());
        String filePath = object.getString(SystemConstant.PATH);
        String type = object.getString(SystemConstant.TYPE);
        if (type.equals(SystemConstant.OSS)) {
            File localPath = new File(rootPath, fileName);
            try {
                File file = ossUtil.ossDownload(filePath, localPath.getPath());
                return file;
            } catch (IOException e) {
                throw ExceptionResultEnum.ERROR.exception("从OSS上下载文件失败");
            }
        } else {
            File file = new File(filePath);
            if (!file.exists()) {
                throw ExceptionResultEnum.ERROR.exception("本地文件不存在");
            }
            return file;
        }
    }

    @Override
    public void downloadFileAndZip(HttpServletResponse response, String rootPath, List<File> files) {
        String schoolId = ServletUtil.getRequestHeaderSchoolId().toString();
        long time = System.nanoTime();
        File rootFile = new File(rootPath);
        // 创建保存目录
        if (!rootFile.exists()) {
            rootFile.mkdirs();
        }
        File zipFile = null;
        try {
            zipFile = FileUtil.file(SystemConstant.TEMP_FILES_DIR + File.separator + schoolId, time + ".zip");
            // 压缩文件
            if (!zipFile.exists()) {
                zipFile.createNewFile();
            }
            File[] srcFiles = files.toArray(new File[files.size()]);
//            ZipUtil.zip(rootFile.getAbsolutePath(), zipFile.getAbsolutePath(), false);

            ZipUtil.zip(zipFile, true, srcFiles);
            outputFile(response, zipFile, String.valueOf(time));
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("下载失败");
        } finally {
            // 删除zip文件
            FileUtil.del(zipFile);
            // 删除压缩内容
            FileUtil.del(rootPath);
        }
    }

    @Override
    public void downloadFileAndZip(HttpServletResponse response, String rootPath, long time) {
        File rootFile = new File(rootPath);
        // 创建保存目录
        if (!rootFile.exists()) {
            rootFile.mkdirs();
        }
        File zipFile = null;
        try {
            zipFile = FileUtil.file(SystemConstant.TEMP_FILES_DIR, time + ".zip");
            // 压缩文件
            if (!zipFile.exists()) {
                zipFile.createNewFile();
            }
            ZipUtil.zip(rootPath, zipFile.getAbsolutePath(), true);
            outputFile(response, zipFile, String.valueOf(time));
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("下载失败");
        } finally {
            // 删除zip文件
            FileUtil.del(zipFile);
            // 删除压缩内容
            FileUtil.del(rootPath);
        }
    }

    @Override
    public void checkData(Long schoolId, String courseCode, String paperNumber, SysUser user) throws IOException {
        // 校验命题任务是否提交
        ExamTask examTask = examTaskService.getByCourseCodeAndPaperNumber(schoolId, courseCode, paperNumber);
        if (examTask != null && !examTask.getEnable()) {
            log.info("命题任务已禁用");
            return;
//            throw ExceptionResultEnum.ERROR.exception("命题任务已禁用");
        }
        ExamTaskDetail examTaskDetail = examTaskDetailService.getByExamTaskId(examTask.getId());
        if (!examTaskDetail.getEnable()) {
            log.info("卷库已禁用");
            return;
        }
        // 1.命题任务已完成
        if (examTask != null && examTask.getStatus().name().equals(ExamStatusEnum.FINISH.name())) {
            // 2.校验考务数据是否导入
            List<ExamDetail> examDetails = examDetailService.listByCourseCodeAndPaperNumber(schoolId, courseCode, paperNumber);
            if (examDetails != null && examDetails.size() > 0) {

                // 3.检查examDetailId下有无其它课程
                for (ExamDetail examDetail : examDetails) {
                    List<ExamDetailCourse> examDetailCourses = examDetailCourseService.listByExamDetailIdAndStatus(examDetail.getId());
                    // 4.没有未完成的命题任务
                    if (examDetailCourses.isEmpty()) {
                        // 更新考场状态为初始新建状态
                        UpdateWrapper<ExamDetail> examDetailUpdateWrapper = new UpdateWrapper<>();
                        examDetailUpdateWrapper.lambda().set(ExamDetail::getStatus, ExamDetailStatusEnum.NEW).eq(ExamDetail::getId, examDetail.getId());
                        examDetailService.update(examDetailUpdateWrapper);

                        //所有考场都撤回，印刷任务状态改为就绪
                        QueryWrapper<ExamDetail> queryWrapper = new QueryWrapper<>();
                        queryWrapper.lambda().eq(ExamDetail::getPrintPlanId, examDetail.getPrintPlanId()).notIn(ExamDetail::getStatus, ExamDetailStatusEnum.NEW, ExamDetailStatusEnum.READY);
                        List<ExamDetail> examDetailss = examDetailService.list(queryWrapper);
                        if (examDetailss.isEmpty()) {
                            UpdateWrapper<ExamPrintPlan> printPlanUpdateWrapper = new UpdateWrapper<>();
                            printPlanUpdateWrapper.lambda().set(ExamPrintPlan::getStatus, PrintPlanStatusEnum.READY).eq(ExamPrintPlan::getId, examDetail.getPrintPlanId());
                            examPrintPlanService.update(printPlanUpdateWrapper);
                        }

                        Map<String, Object> map = tbTaskService.saveTask(TaskTypeEnum.CREATE_PDF, examDetail.getPrintPlanId(), user, examDetail.getId());
                        asyncCreatePdfTempleteService.createPdf(map, null);
                    }
                }
            }
        }
    }

    @Override
    public Set<Long> listSubOrgIds(Long id) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long orgId;
        if (id == null) {
            orgId = sysUser.getOrgId();
        } else {
            orgId = id;
        }
        if (orgId == null) {
            return null;
        }
        List<OrgDto> orgDtos = sysOrgService.listOrgTree();
        Set<Long> stringSet = new HashSet<>();
        stringSet.add(orgId);
        stringSet = getOrgIds(stringSet, orgDtos, orgId);
        return stringSet;
    }

    /**
     * 登录公用
     *
     * @param password
     * @param sysUser
     * @return
     * @throws NoSuchAlgorithmException
     */
    @Override
    public LoginResult login(String password, SysUser sysUser) throws NoSuchAlgorithmException {
        //密码不正确
        if (!Objects.equals(password, sysUser.getPassword())) {
            throw ExceptionResultEnum.PASSWORD_ERROR.exception();
        }
        //停用
        if (!sysUser.getEnable()) {
            throw ExceptionResultEnum.USER_ENABLE.exception();
        }

        Platform platform = ServletUtil.getRequestPlatform();
        String deviceId = ServletUtil.getRequestDeviceId();
        AuthBean authBean = commonService.getUserAuth(sysUser.getId());
        //添加用户鉴权缓存
        if (Objects.isNull(authBean)) {
            throw ExceptionResultEnum.ROLE_ENABLE_AUTHORIZATION.exception();
        }
        //生成token
        String token = SystemConstant.getUuid();
        cacheService.userCache(sysUser.getId());
        //添加用户会话缓存
        Set<RoleTypeEnum> roleType = authBean.getRoleList().stream().map(s -> s.getType()).collect(Collectors.toSet());
        String sessionId = SessionUtil.digest(sysUser.getId(), Math.abs(roleType.toString().hashCode()), platform.name());
        //TODO 测试用
//        String test = SignatureEntityTest.build(SignatureType.TOKEN, sessionId, token);
        ExpireTimeBean expireTime = AuthUtil.getExpireTime(platform);
        TBSession tbSession = new TBSession(sessionId, String.valueOf(sysUser.getId()), roleType.toString(),
                platform.name(), platform.name(), deviceId, ServletUtil.getRequest().getLocalAddr(), token,
                expireTime.getDate().getTime());
        tbSessionService.saveOrUpdate(tbSession);
        redisUtil.setUserSession(sessionId, tbSession, expireTime.getExpireSeconds());

//        LoginResult loginResult = new LoginResult(sysUser, sessionId, test, roleType);
        LoginResult loginResult = new LoginResult(sysUser, sessionId, token, roleType);
        loginResult.setSchoolInfo(Objects.nonNull(authBean.getSchool()) ? loginResult.new SchoolNativeBean(authBean.getSchool()) : null);
        loginResult.setOrgInfo(Objects.nonNull(authBean.getOrg()) ? loginResult.new OrgNativeBean(authBean.getOrg()) : null);
        loginResult.setTime(System.currentTimeMillis());
        return loginResult;
    }

    @Override
    public String createPaperNumber(Long schoolId) {
        String date = DateUtil.today().replace("-", "");
        String paperNumber = convertUtil.getIncre(date, "paperNumber" + schoolId, 5);
        QueryWrapper<ExamTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTask::getSchoolId, schoolId).eq(ExamTask::getPaperNumber, paperNumber);
        ExamTask examTask = examTaskService.getOne(queryWrapper);
        if (examTask != null) {
            return createPaperNumber(schoolId);
        } else {
            return paperNumber;
        }
    }

    @Override
    public Boolean checkExamDetailStatus(Long schoolId, String courseCode, String paperNumber) {
        List<ExamDetail> examDetails = examDetailService.listByCourseCodeAndPaperNumber(schoolId, courseCode, paperNumber);
        if (examDetails != null && examDetails.size() > 0) {
            long count = examDetails.stream().filter(m->!ExamDetailStatusEnum.NEW.name().equals(m.getStatus().name()) && !ExamDetailStatusEnum.READY.name().equals(m.getStatus().name())).count();
            if(count > 0){
                throw ExceptionResultEnum.ERROR.exception("已有印刷任务进入打印状态，不能修改");
            }
        }
        return true;
    }

    private Set<Long> getOrgIds(Set<Long> stringSet, List<OrgDto> orgDtos, Long parentId) {
        for (OrgDto orgDto : orgDtos) {
            Long tempParentId = orgDto.getParentId();
            if (parentId.equals(tempParentId) && stringSet.contains(parentId)) {
                stringSet.add(Long.valueOf(orgDto.getId()));
            }
            if (orgDto.getChildren().isEmpty()) {
                continue;
            }
            getOrgIds(stringSet, orgDto.getChildren(), Long.valueOf(orgDto.getId()));
        }
        return stringSet;
    }

    public static void outputFile(HttpServletResponse response, File file, String fileName) {
        try {

            if (!file.exists()) {
                response.sendError(404, "File not found!");
            }

            BufferedInputStream br = new BufferedInputStream(new FileInputStream(file));
            byte[] buf = new byte[1024];
            int len = 0;

            String fName = new String(fileName.getBytes(), "ISO-8859-1");

            response.reset();
            response.setContentType("application/x-msdownload");
            response.setHeader("Content-Disposition", "attachment; filename=" + fName);

            OutputStream outStream = response.getOutputStream();

            while ((len = br.read(buf)) > 0) {
                outStream.write(buf, 0, len);
            }

            br.close();
            outStream.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
