package com.qmth.distributed.print.business.util;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.Gson;
import com.itextpdf.text.DocumentException;
import com.qmth.distributed.print.business.bean.dto.ExamStudentDto;
import com.qmth.distributed.print.business.bean.dto.PaperPdfDto;
import com.qmth.distributed.print.business.bean.dto.PdfDto;
import com.qmth.distributed.print.business.cache.CreatePdfCacheUtil;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.*;
import com.qmth.distributed.print.business.service.BasicAttachmentService;
import com.qmth.distributed.print.business.service.CommonService;
import com.qmth.distributed.print.business.service.ExamDetailService;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 创建pdf util
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/4/14
 */
@Component
public class CreatePdfUtil {
    private final static Logger log = LoggerFactory.getLogger(CreatePdfUtil.class);

    @Resource
    OssUtil ossUtil;

    @Resource
    BasicAttachmentService basicAttachmentService;

    @Resource
    FreemarkerUtil freemarkerUtil;

    @Resource
    CommonService commonService;

    @Resource
    ExamDetailService detailService;

    @Resource
    RedisUtil redisUtil;

    /**
     * 创建登记表
     *
     * @param examDetail
     * @param basicAttachment
     * @param ordinaryPdfList
     * @param printCount
     */
    public void createCheckIn(ExamDetail examDetail, BasicAttachment basicAttachment, List<PdfDto> ordinaryPdfList, Integer printCount) throws IOException, DocumentException {
        if (Objects.isNull(basicAttachment)) {
            throw ExceptionResultEnum.ERROR.exception("找不到附件");
        }
        String type = basicAttachment.getType();
        JSONObject jsonObject = JSONObject.parseObject(basicAttachment.getPath());
        if (Objects.nonNull(type) && Objects.equals(type, SystemConstant.PDF_PREFIX)) {
            String filePath = (String) jsonObject.get(SystemConstant.PATH);
            String url = SystemConstant.TEMP_FILES_DIR + File.separator + filePath;
            File localFile = ossUtil.ossDownload(filePath, url);
            PdfDto pdfDto = PdfUtil.addPdfPage(localFile);
            for (int i = 0; i < printCount; i++) {
                ordinaryPdfList.add(new PdfDto(localFile.getPath(), PageSizeEnum.A4, pdfDto.getPageCount()));
            }
        } else {
            basicAttachmentService.saveAttachmentPdf(ClassifyEnum.CHECK_IN, examDetail, basicAttachment, ordinaryPdfList, printCount, 0);
        }
    }

    /**
     * 生成卷袋贴
     *
     * @param tag
     * @param basicAttachment
     * @param schoolName
     * @param examDetailCourse
     * @param examStudentList
     * @param variablePdfList
     * @param printCount
     * @throws IOException
     */
    public void createPaperPackage(boolean tag, BasicAttachment basicAttachment, String schoolName, ExamDetail examDetail, ExamDetailCourse examDetailCourse, List<ExamStudent> examStudentList, List<PdfDto> variablePdfList, Integer printCount) throws IOException {
        if (Objects.isNull(basicAttachment)) {
            throw ExceptionResultEnum.ERROR.exception("找不到附件");
        }
        Map<String, Object> htmlMap = new HashMap<>();
        htmlMap.put("examDetailCourseId", examDetailCourse.getId());
        htmlMap.put("schoolName", schoolName);
        htmlMap.put("courseName", examDetailCourse.getCourseName());
        htmlMap.put("courseCode", examDetailCourse.getCourseCode());
        htmlMap.put("examSite", examDetail.getExamPlace());
        htmlMap.put("examRoom", examDetail.getExamRoom());
        htmlMap.put("startTime", DateUtil.format(new Date(examDetail.getExamStartTime()), SystemConstant.DEFAULT_DATE_PATTERN));
        htmlMap.put("endTime", DateUtil.format(new Date(examDetail.getExamEndTime()), SystemConstant.DEFAULT_DATE_PATTERN));
        htmlMap.put("paperCode", examDetail.getPackageCode());
        if (Objects.nonNull(examDetailCourse.getPaperNumber())) {
            htmlMap.put("paperCodeImg", GoogleBarCodeUtil.createBarCode(examDetail.getPackageCode(), false));
        }

        List<String> extendColumnList = examStudentList.stream().map(m -> m.getExtendFields()).distinct().collect(Collectors.toList());
        Set<String> startCollege = new HashSet();
        Set<String> className = new HashSet();
        Set<String> proctorCollege = new HashSet();
        Set<String> examManager = new HashSet();

        if (Objects.nonNull(extendColumnList) && extendColumnList.size() > 0) {
            for (String s : extendColumnList) {
                List<Map> mapList = JSONObject.parseArray(s, Map.class);
                for (Map map : mapList) {
                    if (Objects.equals("startCollege", map.get("code").toString())) {
                        startCollege.add(map.get("value").toString());
                    }
                    if (Objects.equals("className", map.get("code").toString())) {
                        className.add(map.get("value").toString());
                    }
                    if (Objects.equals("proctorCollege", map.get("code").toString())) {
                        proctorCollege.add(map.get("value").toString());
                    }
                    if (Objects.equals("examManager", map.get("code").toString())) {
                        examManager.add(map.get("value").toString());
                    }
                }
            }
        }

        htmlMap.put("startCollege", String.join(",", startCollege));
        htmlMap.put("className", String.join(",", className));
        htmlMap.put("proctorCollege", String.join(",", proctorCollege));
        htmlMap.put("examManager", String.join(",", examManager));

        List<ExamStudent> tempList = examStudentList.stream().filter(s -> (Objects.nonNull(s.getSiteNumber()) && !Objects.equals("", s.getSiteNumber().trim()))).collect(Collectors.toList());
        String minSite = "", maxSite = "";
        if (Objects.nonNull(tempList) && tempList.size() > 0) {
            minSite = tempList.stream().min((a, b) -> a.getSiteNumber().compareTo(b.getSiteNumber())).get().getSiteNumber();
            maxSite = tempList.stream().max((a, b) -> a.getSiteNumber().compareTo(b.getSiteNumber())).get().getSiteNumber();
        }

        htmlMap.put("minSite", minSite);
        htmlMap.put("maxSite", maxSite);
        htmlMap.put("studentCount", examStudentList.size());

        htmlMap.put("tag", tag ? "订" : null);

        htmlMap.computeIfAbsent("basicAttachment", v -> basicAttachment);
        htmlMap.computeIfAbsent("variablePdfList", v -> variablePdfList);
        htmlMap.computeIfAbsent("printCount", v -> printCount);
        htmlMap.computeIfAbsent("sequence", v -> 1);
        htmlMap.computeIfAbsent("examDetail", v -> examDetail);
        htmlMap.computeIfAbsent("printType", v -> ClassifyEnum.PACKAGE);
        freemarkerUtil.createPaperPackage(htmlMap);
    }

    /**
     * 创建签到表
     *
     * @param basicAttachment
     * @param schoolName
     * @param examDetail
     * @param examDetailCourse
     * @param examStudentList
     * @param variablePdfList
     * @param printCount
     */
    public void createSignBook(BasicAttachment basicAttachment, String schoolName, ExamDetail examDetail, ExamDetailCourse examDetailCourse, List<ExamStudent> examStudentList, List<PdfDto> variablePdfList, Integer printCount) throws IOException {
        if (Objects.isNull(basicAttachment)) {
            throw ExceptionResultEnum.ERROR.exception("找不到附件");
        }
        Map<String, Object> htmlMap = new HashMap<>();
        htmlMap.put("examDetailCourseId", examDetailCourse.getId());
        htmlMap.put("schoolName", schoolName);
        htmlMap.put("courseName", examDetailCourse.getCourseName());
        htmlMap.put("courseCode", examDetailCourse.getCourseCode());
        htmlMap.put("examRoom", examDetail.getExamRoom());

        List<String> extendColumnList = examStudentList.stream().map(m -> m.getExtendFields()).distinct().collect(Collectors.toList());
        Set<String> classTeacher = new HashSet();
        if (Objects.nonNull(extendColumnList) && extendColumnList.size() > 0) {
            for (String s : extendColumnList) {
                List<Map> mapList = JSONObject.parseArray(s, Map.class);
                for (Map map : mapList) {
                    if (Objects.equals("classTeacher", map.get("code").toString())) {
                        classTeacher.add(map.get("value").toString());
                    }
                }
            }
        }
        htmlMap.put("classTeacher", String.join(",", classTeacher));
        htmlMap.put("startTime", DateUtil.format(new Date(examDetail.getExamStartTime()), SystemConstant.DEFAULT_DATE_PATTERN));
        htmlMap.put("endTime", DateUtil.format(new Date(examDetail.getExamEndTime()), SystemConstant.DEFAULT_DATE_PATTERN));
        htmlMap.put("paperCode", examDetail.getPackageCode());
        if (Objects.nonNull(examDetailCourse.getPaperNumber())) {
            htmlMap.put("paperCodeImg", GoogleBarCodeUtil.createBarCode(examDetail.getPackageCode(), false));
        }

        int totalCount = examStudentList.size();
        if (totalCount > 0) {
            List<Map> subList = new ArrayList<>();
            int pageCount = totalCount % 80 == 0 ? totalCount / 80 : totalCount / 80 + 1;
            for (int i = 0; i < pageCount; i++) {
                Map subMap = new HashMap();
                subMap.put("index", i + 1);
                int studentCount;
                List<ExamStudent> subStudents;
                if (pageCount == 1) {
                    studentCount = totalCount;
                    subStudents = examStudentList;
                } else if (pageCount > 1 && i == pageCount - 1) {
                    studentCount = totalCount - (pageCount - 1) * 80;
                    subStudents = examStudentList.subList(80 * i, examStudentList.size());
                } else {
                    studentCount = 80;
                    subStudents = examStudentList.subList(80 * i, 80 * (i + 1));
                }
                subMap.put("studentCount", studentCount);
                List<ExamStudentDto> examStudentList1 = new ArrayList<>();
                List<ExamStudentDto> examStudentList2 = new ArrayList<>();

                Gson gson = new Gson();
                int mod = subStudents.size() % 2;
                int htmlTableCount = mod == 0 ? subStudents.size() / 2 : subStudents.size() / 2 + 1;
                for (int j = 0; j < htmlTableCount; j++) {
                    ExamStudentDto examStudentDto = gson.fromJson(gson.toJson(subStudents.get(j)), ExamStudentDto.class);
                    examStudentDto.setIndex(j + 1);
                    examStudentDto.setExtendFields(getClassName(examStudentDto.getExtendFields()));
                    examStudentList1.add(examStudentDto);
                }
                for (int j = htmlTableCount; j < subStudents.size(); j++) {
                    ExamStudentDto examStudentDto = gson.fromJson(gson.toJson(subStudents.get(j)), ExamStudentDto.class);
                    examStudentDto.setIndex(j + 1);
                    examStudentDto.setExtendFields(getClassName(examStudentDto.getExtendFields()));
                    examStudentList2.add(examStudentDto);
                }
                if (examStudentList1.size() > examStudentList2.size()) {
                    for (int j = subStudents.size() - htmlTableCount; j < examStudentList1.size(); j++) {
                        examStudentList2.add(new ExamStudentDto());
                    }
                }
                subMap.put("studentList1", examStudentList1);
                subMap.put("studentList2", examStudentList2);
                subList.add(subMap);
            }
            htmlMap.put("subList", subList);
        }
        htmlMap.computeIfAbsent("basicAttachment", v -> basicAttachment);
        htmlMap.computeIfAbsent("variablePdfList", v -> variablePdfList);
        htmlMap.computeIfAbsent("printCount", v -> printCount);
        htmlMap.computeIfAbsent("sequence", v -> 2);
        htmlMap.computeIfAbsent("examDetail", v -> examDetail);
        htmlMap.computeIfAbsent("printType", v -> ClassifyEnum.SIGN);
        freemarkerUtil.createSignBook(htmlMap);
    }

    /**
     * 替换html通用模版
     *
     * @param examCardDetail
     * @return
     */
    public String replaceHtmlTemplete(ExamCardDetail examCardDetail) {
        String content = examCardDetail.getHtmlContent();
        log.info("contentTemp:{}", content);
        content = content.replaceAll("<img src=\"data:image/png;base64,\\$\\{examNumber\\}\">", "");
        content = content.replaceAll("\\$\\{examNumberStr\\}", "");
        return content;
    }

    /**
     * 获取题卡attachmentId
     *
     * @param examCardDetail
     * @param attachmentIds
     */
    public void getCardAttachmentId(ExamCardDetail examCardDetail, Set<Long> attachmentIds) {
        if (Objects.nonNull(examCardDetail.getAttachmentId()) && !Objects.equals("", examCardDetail.getAttachmentId())) {
            JSONObject jsonObjectCard = JSONObject.parseObject(examCardDetail.getAttachmentId());
            JSONArray jsonArrayCard = (JSONArray) jsonObjectCard.get("card");
            for (int i = 0; i < jsonArrayCard.size(); i++) {
                JSONObject object = (JSONObject) jsonArrayCard.get(i);
                if (Objects.nonNull(object.get("attachmentId")) && !Objects.equals("", object.get("attachmentId"))) {
                    attachmentIds.add((Long) (object.get("attachmentId")));
                }
            }
        }
    }

    /**
     * 获取试卷pdf
     *
     * @param paperPdfDto
     * @param pdfList
     * @return
     * @throws IOException
     */
    public PdfDto getExamStudentPaperPdf(PaperPdfDto paperPdfDto, List<PdfDto> pdfList) throws IOException {
        if (Objects.nonNull(paperPdfDto)) {
            int pages = paperPdfDto.getPages();
            PdfDto pdfDto = PdfUtil.addPdfPage(paperPdfDto.getFile());
            pdfList.add(new PdfDto(paperPdfDto.getFile().getPath(), PageSizeEnum.A3, pdfDto.getPageCount()));
            return new PdfDto(PageSizeEnum.A3, pages);
        } else {
            return null;
        }
    }

    /**
     * 获取试卷pdf
     *
     * @param paperPdfDto
     * @param backupCount
     * @param pdfList
     * @return
     * @throws IOException
     */
    public PdfDto getPaperPdf(PaperPdfDto paperPdfDto, Integer backupCount, List<PdfDto> pdfList) throws IOException {
        if (Objects.nonNull(paperPdfDto)) {
            int pages = paperPdfDto.getPages();
            boolean tag = pages > 2 ? true : false;
            PdfDto pdfDto = PdfUtil.addPdfPage(paperPdfDto.getFile());
            for (int j = 1; j <= backupCount; j++) {
                pdfList.add(new PdfDto(paperPdfDto.getFile().getPath(), PageSizeEnum.A3, pdfDto.getPageCount()));
            }
            return new PdfDto(PageSizeEnum.A3, pages, tag);
        } else {
            return null;
        }
    }

    /**
     * 获取考试试卷
     *
     * @param paperType
     * @param examTaskDetail
     * @return
     * @throws IOException
     */
    public PaperPdfDto getPaperPdfFile(String paperType, ExamTaskDetail examTaskDetail) throws IOException {
        PaperPdfDto paperPdfDto = null;
        JSONArray jsonArrayPaper = JSONArray.parseArray(examTaskDetail.getPaperAttachmentIds());
        for (int i = 0; i < jsonArrayPaper.size(); i++) {
            JSONObject object = (JSONObject) jsonArrayPaper.get(i);
            if (Objects.nonNull(object.get("attachmentId")) && !Objects.equals("", object.get("attachmentId"))) {
                Long attachmentId = Long.parseLong((String) (object.get("attachmentId")));
                BasicAttachment basicAttachment = basicAttachmentService.getById(attachmentId);
                if (Objects.isNull(basicAttachment)) {
                    throw ExceptionResultEnum.ERROR.exception("找不到附件");
                }
                String name = (String) object.get("name");
                if (Objects.equals(name.toUpperCase(), paperType.toUpperCase())) {
                    File file = commonService.getFile(basicAttachment.getPath(), false);
                    int pages = (int) object.get("pages");
                    paperPdfDto = new PaperPdfDto(file, pages);
                    break;
                }
            }
        }
        return paperPdfDto;
    }

    /**
     * 抽取试卷
     *
     * @param examPrintPlan
     * @param examTaskDetail
     * @param key
     * @return
     */
    public String getPaperType(ExamPrintPlan examPrintPlan, ExamTaskDetail examTaskDetail, String key) {
        //抽取卷型
        DrawRuleEnum drawRule = Objects.nonNull(examPrintPlan.getDrawRule()) ? examPrintPlan.getDrawRule() : DrawRuleEnum.ONE;
        //未曝光卷型
        String unexposedPaperType = examTaskDetail.getUnexposedPaperType();
        //已曝光卷型
        String exposedPaperType = examTaskDetail.getExposedPaperType();
        if (drawRule == DrawRuleEnum.ONE) {
            if (Objects.isNull(unexposedPaperType)) {
                CreatePdfCacheUtil.deletePaperType(key);
                throw ExceptionResultEnum.ERROR.exception("当前没有未曝光的卷型");
            }
        } else {
            if (Objects.isNull(exposedPaperType) && Objects.isNull(unexposedPaperType)) {
                CreatePdfCacheUtil.deletePaperType(key);
                throw ExceptionResultEnum.ERROR.exception("当前没有未曝光的卷型");
            }
        }
        String paperType = null;
        String[] paperTypes = null;
        boolean lock = true;
        for (int i = 0; i < SystemConstant.MAX_RETRY_COUNT; i++) {
            lock = redisUtil.lock(key, SystemConstant.REDIS_CACHE_TIME_OUT);
            if (lock) {
                try {
                    paperType = CreatePdfCacheUtil.getPaperType(key);
                    if (Objects.isNull(paperType) || !unexposedPaperType.contains(paperType)) {
                        if (drawRule == DrawRuleEnum.ONE) {
                            paperTypes = unexposedPaperType.split(",");
                        } else {
                            if (Objects.nonNull(unexposedPaperType)) {
                                paperTypes = unexposedPaperType.split(",");
                            } else {
                                paperTypes = exposedPaperType.split(",");
                            }
                        }
                        int paperRandom = new Random().nextInt(paperTypes.length);
                        paperType = paperTypes[paperRandom];
                        CreatePdfCacheUtil.setPaperType(key, paperType);
                    }
                    break;
                } catch (Exception e) {
                    log.error("请求出错", e);
                } finally {
                    redisUtil.releaseLock(key);
                }
            } else {//未获取到锁，等待500毫秒继续获取
                try {
                    Thread.sleep(500);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
        log.info("getPaperType key:{},paperType:{},lock:{}", key, paperType, lock);
        if (!lock) {
            log.info("getPaperType 未获取到锁,key:{}", key);
            throw ExceptionResultEnum.ERROR.exception("未获取到试卷类型");
        }
        return paperType;
    }

    /**
     * 合并pdf
     *
     * @param tbTask
     * @param examDetail
     * @param userId
     * @param schoolId
     * @param list
     * @return
     * @throws IOException
     */
    public BasicAttachment mergePdf(BasicExamRule basicExamRule, TBTask tbTask, ExamDetail examDetail, Long userId, Long schoolId, List<PdfDto>... list) throws IOException {
        StringJoiner stringJoiner = new StringJoiner("").add(SystemConstant.PDF_TEMP_FILES_DIR).add(File.separator);
        List<PdfDto> mergePdfA3List = new ArrayList<>();
        List<PdfDto> mergePdfA4List = new ArrayList<>();
        List<PdfDto> mergePdfDeleteList = new ArrayList<>();
//        AtomicInteger pageA4Count = new AtomicInteger(0);
//        AtomicInteger pageA3Count = new AtomicInteger(0);
        for (int i = 0; i < list.length; i++) {
            if (list[i].size() > 0 && list[i].get(0).getPageSize() == PageSizeEnum.A4) {
//                pageA4Count.addAndGet(list[i].stream().mapToInt(PdfDto::getPageCount).sum());
                mergePdfA4List.addAll(list[i]);
            } else if (list[i].size() > 0 && list[i].get(0).getPageSize() == PageSizeEnum.A3) {
//                pageA3Count.addAndGet(list[i].stream().mapToInt(PdfDto::getPageCount).sum());
                mergePdfA3List.addAll(list[i]);
            }
        }
//        mergePdfDeleteA3List.addAll(list[2]);
        mergePdfDeleteList.addAll(list[3]);
        mergePdfDeleteList.addAll(list[5]);
        List<String> pathA4List = mergePdfA4List.stream().map(PdfDto::getPath).collect(Collectors.toList());
        List<String> pathA3List = mergePdfA3List.stream().map(PdfDto::getPath).collect(Collectors.toList());
        String dirNameA4 = PdfUtil.mergePdf(pathA4List.toArray(new String[mergePdfA4List.size()]), null);
        String dirNameA3 = PdfUtil.mergePdf(pathA3List.toArray(new String[mergePdfA3List.size()]), null);
        // oss上只认"/"，windows生成的路径分隔符全部替换为"/"
        File localA4PdfFile = null, localA3PdfFile = null;
        BasicAttachment basicAttachment = null;
        if (Objects.nonNull(dirNameA4)) {
            dirNameA4 = dirNameA4.replaceAll("\\\\", "/");
            localA4PdfFile = new File(stringJoiner.toString() + File.separator + dirNameA4);
        }
        if (Objects.nonNull(dirNameA3)) {
            dirNameA3 = dirNameA3.replaceAll("\\\\", "/");
            localA3PdfFile = new File(stringJoiner.toString() + File.separator + dirNameA3);
            basicAttachment = basicAttachmentService.saveAttachmentPdf(dirNameA3, userId);
            tbTask.setImportFileName(basicAttachment.getName());
            tbTask.setImportFilePath(basicAttachment.getPath());
            examDetail.setAttachmentId(basicAttachment.getId());
        }
        examDetail.setPagesA3(PdfUtil.getPdfPages(localA3PdfFile));
        examDetail.setPagesA4(PdfUtil.getPdfPages(localA4PdfFile));
//        log.info("pageA3Count:{},examDetail:{}", pageA3Count.get(), JacksonUtil.parseJson(examDetail));
        examDetail.setStatus(PrintMethodEnum.AUTO == basicExamRule.getPrintMethod() ? ExamDetailStatusEnum.WAITING : ExamDetailStatusEnum.READY);
        detailService.saveOrUpdate(examDetail);
        if (Objects.nonNull(localA3PdfFile)) {
            ossUtil.ossUpload(dirNameA3, localA3PdfFile, DigestUtils.md5Hex(new FileInputStream(localA3PdfFile)));
            localA3PdfFile.delete();
        }
        if (Objects.nonNull(localA4PdfFile)) {
            localA4PdfFile.delete();
        }
        //删除题卡临时文件
        for (PdfDto pdfDto : mergePdfDeleteList) {
            if (Objects.nonNull(pdfDto)) {
                new File(pdfDto.getPath()).delete();
            }
        }
//        for (PdfDto pdfDto : mergePdfA3List) {
//            if (Objects.nonNull(pdfDto)) {
//                new File(pdfDto.getPath()).delete();
//            }
//        }
//        for (PdfDto pdfDto : mergePdfA4List) {
//            if (Objects.nonNull(pdfDto)) {
//                new File(pdfDto.getPath()).delete();
//            }
//        }
        return basicAttachment;
    }

    /**
     * 删除附件
     *
     * @param attachmentIds
     * @param ftlList
     */
    public void deleteAttachment(Set<Long> attachmentIds, Set<File> ftlList) {
        if (Objects.nonNull(attachmentIds) && attachmentIds.size() > 0) {
            QueryWrapper<BasicAttachment> basicAttachmentQueryWrapper = new QueryWrapper<>();
            basicAttachmentQueryWrapper.lambda().in(BasicAttachment::getId, attachmentIds);
            basicAttachmentService.batchDeleteAttachment(basicAttachmentService.list(basicAttachmentQueryWrapper));
            basicAttachmentService.removeByIds(attachmentIds);
        }
        if (Objects.nonNull(ftlList) && ftlList.size() > 0) {
            for (File file : ftlList) {
                file.delete();
            }
        }
    }

    /**
     * 考生题卡html
     *
     * @param attachmentIds
     * @param studentContent
     * @param t
     * @param examDetail
     * @param examDetailCourse
     * @param userId
     * @param examStudentPdfList
     * @return
     * @throws IOException
     */
    public BasicAttachment examStudentHtml(Set<Long> attachmentIds, String studentContent, ExamStudent t, ExamDetail examDetail, ExamDetailCourse examDetailCourse, Long userId, List<PdfDto> examStudentPdfList) throws IOException {
        if (Objects.nonNull(t.getAttachmentId())) {
            attachmentIds.add(t.getAttachmentId());
        }
        String studentHtml = studentContent;
        if (Objects.nonNull(t.getExtendFields())) {
            JSONArray jsonObjectExtend = (JSONArray) JSONArray.parse(t.getExtendFields());//扩展字段
            if (Objects.nonNull(jsonObjectExtend)) {
                for (int i = 0; i < jsonObjectExtend.size(); i++) {
                    JSONObject object = (JSONObject) jsonObjectExtend.get(i);
                    studentHtml = studentHtml.replaceAll("\\$\\{" + object.get("code") + "\\}", String.valueOf(object.get("value")));
                }
            }
        }
        //生成学生考号条码并将图片转成base64
        if (Objects.nonNull(t.getTicketNumber())) {
            studentHtml = studentHtml.replaceAll("\\$\\{examNumber\\}", GoogleBarCodeUtil.createBarCode(t.getTicketNumber(), false));
        }
        studentHtml = studentHtml.replaceAll("\\$\\{examNumberStr\\}", t.getTicketNumber());
        //随机生成学生试卷条码并将图片转成base64
        if (Objects.nonNull(examDetailCourse.getPaperType())) {
            studentHtml = studentHtml.replaceAll("\\$\\{paperType\\}", GoogleBarCodeUtil.createBarCode(SystemConstant.convertPaperType(examDetailCourse.getPaperType()), false));
        }
        Map<String, Object> map = ConvertUtil.analyzeDateAndTime(examDetail.getExamStartTime(), examDetail.getExamEndTime());
        studentHtml = studentHtml.replaceAll("\\$\\{examDate\\}", Objects.nonNull(map.get("date")) ? (String) map.get("date") : "");
        studentHtml = studentHtml.replaceAll("\\$\\{examTime\\}", Objects.nonNull(map.get("time")) ? (String) map.get("time") : "");
        studentHtml = studentHtml.replaceAll("\\$\\{ticketNumber\\}", t.getTicketNumber());
        studentHtml = studentHtml.replaceAll("\\$\\{siteNumber\\}", t.getSiteNumber());
        studentHtml = studentHtml.replaceAll("\\$\\{paperTypeName\\}", examDetailCourse.getPaperType());
        studentHtml = studentHtml.replaceAll("\\$\\{studentCode\\}", t.getStudentCode());
        studentHtml = studentHtml.replaceAll("\\$\\{studentName\\}", t.getStudentName());
        studentHtml = studentHtml.replaceAll("\\$\\{courseName\\}", examDetailCourse.getCourseName());
        studentHtml = studentHtml.replaceAll("\\$\\{courseCode\\}", examDetailCourse.getCourseCode());
        studentHtml = studentHtml.replaceAll("\\$\\{examPlace\\}", examDetail.getExamPlace());
        studentHtml = studentHtml.replaceAll("\\$\\{examRoom\\}", examDetail.getExamRoom());
        studentHtml = studentHtml.replaceAll("\\$\\{paperNumber\\}", examDetailCourse.getPaperNumber());

        //学生题卡
        BasicAttachment examStudentAttachment = basicAttachmentService.saveAttachmentHtml(examDetail.getSchoolId() + "|" + examDetailCourse.getCourseCode() + "|" + t.getTicketNumber(), studentHtml, userId, examStudentPdfList);
        t.setAttachmentId(examStudentAttachment.getId());
        t.setPaperType(examDetailCourse.getPaperType());
        return examStudentAttachment;
    }

    /**
     * 通用题卡html
     *
     * @param cardContent
     * @param examDetailCourse
     * @param examCard
     * @param jsonArray
     * @param userId
     * @param cardPdfList
     * @return
     * @throws IOException
     */
    public BasicAttachment cardHtml(String sequence, String cardContent, ExamDetail examDetail, ExamDetailCourse examDetailCourse, ExamCard examCard, JSONArray jsonArray, Long userId, List<PdfDto> cardPdfList) throws IOException {
        //通用题卡
        String cardTemp = cardContent;
        cardTemp = cardTemp.replaceAll("\\$\\{paperTypeName\\}", examDetailCourse.getPaperType());
        //随机生成试卷条码并将图片转成base64
        if (Objects.nonNull(examDetailCourse.getPaperType())) {
            cardTemp = cardTemp.replaceAll("\\$\\{paperType\\}", GoogleBarCodeUtil.createBarCode(SystemConstant.convertPaperType(examDetailCourse.getPaperType()), false));
        }
        //通用题卡生成卷袋贴条码
        String packageCode = examDetail.getPackageCode() + sequence;
        String packageCodeImg = null;
        if (Objects.nonNull(packageCode)) {
            packageCodeImg = GoogleBarCodeUtil.createBarCode(packageCode, false);
        }
        String packageCodeDiv = "<div class=\"page-box page-box-0\"><div class=\"package-number\" style=\"position: absolute;width: 200px;height: 40px;top: 10px;right: 25%;margin-left: -100px;text-align: center;z-index: 99;\"><img src=\"data:image/png;base64," + packageCodeImg + "\" style=\"display: block; height: 28px; width: 100%\" /><p style=\"line-height: 1; font-size: 12px; margin: 0;\">" + packageCode + "</p></div>";
        cardTemp = cardTemp.replaceAll("<div class=\"page-box page-box-0\">", packageCodeDiv);
        BasicAttachment cardAttachment = basicAttachmentService.saveAttachmentHtml(examCard.getSchoolId() + "|" + examCard.getCourseCode(), cardTemp, userId, cardPdfList);
        JSONObject object = new JSONObject();
        object.put("name", examDetailCourse.getPaperType());
        object.put("examDetailCourseId", examDetailCourse.getId());
        object.put("attachmentId", cardAttachment.getId());
        object.put("packageCode", packageCode);
        jsonArray.add(object);
        return cardAttachment;
    }

    /**
     * 获取班级名称
     *
     * @param extendCloumn
     * @return
     */
    private String getClassName(String extendCloumn) {
        List<Map> mapList = JSONObject.parseArray(extendCloumn, Map.class);
        for (Map map : mapList) {
            if (Objects.equals("className", map.get("code").toString())) {
                return map.get("value").toString();
            }
        }
        return null;
    }
}
